@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
-#from datetime import datetime
+from datetime import datetime
 import copy
-import urlparse
 import slumber
+import urlparse
 
 
 __all__ = ["Client"]
@@ -20,146 +20,80 @@ class FieldTypeError(TypeError):
     pass
 
 
-def parse_id(resouce_uri):
-    """ url parsing
-
-    :param resource_uri:
-    :rtype: str
-    :return: primary id
-    """
-    return resouce_uri.split("/")[::-1][1]
-
-
-class Response(object):
-    """ Proxy Model Class """
-    def __init__(self, model, response=None):
-        """
-
-        :param model: The Model
-        :param response: response from Client Library
-        """
-        response = response or dict()
-        self.__dict__["_response"] = response
-        self.model = model(**response)
-        self._schema = self.model.schema()
-        self._url = ""
-
-    def _get_res(self):
-        return self.__dict__["_response"]
-
-    def _set_res(self, value):
-        self.__dict__["_response"] = value
-
-    def _get_response(self):
-        return self._res
-
-    def _set_response(self, value):
-        self._res = value
-
-    _res = property(_get_res, _set_res)
-    _response = property(_get_response, _set_response)
-
-    def __repr__(self):
-        return "<{0}: {1} {2}>".format(self.model._model_name, self._url, self._res)
-
-    def __getattr__(self, attr):
-        """ return Response Class """
-        if not attr in self._response:
-            raise AttributeError(attr)
-        elif not "related_type" in self._schema["fields"][attr]:
-            return self._response[attr]
-
-        related_type = self._schema["fields"][attr]["related_type"]
-        if related_type == "to_many":
-            return ManyToManyManager(model=self.model,
-                            query={"id__in": [parse_id(url) for url in self._response[attr]]})
-        elif related_type == "to_one":
-            return LazyResponse(model=self.model, model_name=attr, url=self._response[attr])
-
-    def __getitem__(self, item):
-        if item in self._response:
-            return self._response[item]
-        else:
-            raise KeyError(item)
-
-    def __contains__(self, attr):
-        return attr in self._res
-
-    def __setattr__(self, attr, value):
-        if self.__contains__(attr):
-            self._res[attr] = value
-            self.model.__setattr__(attr, value)
-        super(Response, self).__setattr__(attr, value)
-
-    def save(self):
-        """ save saved response """
-        self.model.save()
-
-    def delete(self):
-        """ remove saved response """
-        self.model.delete()
-        self._response = dict()
-
-
-class LazyResponse(Response):
-    """ convert response model and lazy response """
-
-    def __init__(self, model_name, url, *args, **kwargs):
-        super(LazyResponse, self).__init__(*args, **kwargs)
-        self.model = self.model._clone(model_name)
-        self._client = getattr(self.model._main_client, model_name)
-        self._schema = self.model.schema()
-        self._url = url
-
-    def _get_response(self):
-        if not self._res:
-            self._res = self._client(parse_id(self._url)).get()
-        return self._res
-
-    def _set_response(self, value):
-        self._res = value
-
-    _response = property(_get_response, _set_response)
-
-
 class QuerySet(object):
 
     def __init__(self, model, responses=None, query=None, **kwargs):
         self.model = model
         self._kwargs = kwargs
         self._query = query or dict()
+        self._iteration_num = None
         self._response_class = kwargs.get("response_class", Response)
         self._set_objects(responses)  # set _responses, _meta, _objects, _objects_count
-        self._iteration_num = None
 
     def __repr__(self):
+        if self.__objects is None:
+            numerator = 0
+        else:
+            numerator = len(self._objects)
         return "<QuerySet {0} ({1}/{2})>".format(
-                    self._response_class, self._objects_count, len(self))
+                    self._response_class, numerator, len(self))
 
     def __len__(self):
         """ total count """
-        return self.count()
+        if self._iteration_num is None:
+            return self.count()
+        return self._iteration_num
 
     def __iter__(self):
+        if len(self._objects) < 1\
+        or len(self._objects) < len(self):
+            return self._iteration()
+        return iter(self._objects)
+
+    def _iteration(self):
         if len(self) < 1:
             raise StopIteration()
-        index = 0
-        length = 0
-        klass = copy.deepcopy(self)
+        index, length = 0, 0
+        klass = self._clone(self._responses)  #  klass = copy.deepcopy(self)
         while 1:
             try:
-                yield klass._wrap_response(klass._objects[index])
+                yield klass._objects[index]
                 index += 1
                 length += 1
-            except KeyError:
-                if self._iteration_num <= length and self._iteration_num is not None:
+            except IndexError:
+                if self._iteration_num <= length \
+                and self._iteration_num is not None:
                     raise StopIteration()
                 klass = klass._next()
+                self._objects.extend(klass._objects)
                 index = 0
 
+    def __get_objects(self):
+        if self.__objects is not None \
+        and isinstance(self.__objects, (tuple, list)):
+            for i, obj in enumerate(self.__objects):
+                if isinstance(obj, dict) is True:
+                    self.__objects[i] = self._wrap_response(obj)
+            return self.__objects
+        else:
+            return []
+
+    def __set_objects(self, value):
+        self.__objects = value
+
+    _objects = property(__get_objects, __set_objects)
+
+    def _fill_objects(self):
+        self._set_objects(self._get_responses())
+
+    def _set_objects(self, responses):
+        self._responses = responses
+        self._meta = responses and responses["meta"]
+        self._objects = responses and responses["objects"]
+
     def _clone(self, responses=None, klass=None, **kwargs):
-        klass = klass or self.__class__
         responses = responses or self._responses
+        klass = klass or self.__class__
 
         clone = klass(model=self.model, responses=responses, query=self._query)
         clone.__dict__.update(kwargs)
@@ -198,19 +132,10 @@ class QuerySet(object):
 
             if not self._responses:
                 self._fill_objects()
-            return self._wrap_response(self._objects[index])
+            return self._objects[index]
         except KeyError as err:
             raise IndexError(err)
 
-    def _fill_objects(self):
-        self._set_objects(self._get_responses())
-
-    def _set_objects(self, responses):
-        self._responses = responses
-        self._meta = responses and responses["meta"]
-        self._objects = dict(enumerate(responses["objects"])) if responses else []
-        self._objects_count = len(self._objects)
-
     def _get_responses(self, **kwargs):
         return self.model._client.get(**kwargs)
 
@@ -290,25 +215,19 @@ class QuerySet(object):
         return self._filter(*args, **kwargs)
 
     def _get_ids(self):
-        return [parse_id(self._objects[i]["resource_uri"]) for i in self._objects]
+        return [parse_id(obj["resource_uri"]) for obj in self._objects]
 
     def _filter(self, *args, **kwargs):
-
         # TODO: ↓↓↓ ManyToManyで 一件も relationがない場合の処理, 現状元のQuerySetの結果が返される ↓↓↓↓
         # <QuerySet <class 'queryset_client.client.Response'> (0/0)>
-
         # TODO: id__in 上書きされる
-
         query = dict(self._query.items() + kwargs.items())
         clone = self._clone(self._get_responses(**query))
         clone._query.update({"id__in": clone._get_ids()})
-
         return clone
 
     def order_by(self, *args, **kwargs):
-
         # TODO: multiple order_by = "order_by=-body&order_by=id"
-
         order = {"order_by": args[0]}
         clone = self._filter(*args, **dict(order.items() + kwargs.items()))
         clone._query.update(order)
@@ -425,174 +344,274 @@ class ManyToManyManager(Manager):
         #  TODO: ManyToMany Manager  parent_obj.add(related_object)
         pass
 
-class Model(object):
+    def filter(self, *args, **kwargs):
+        if "id__in" in kwargs:
+            raise NotImplementedError("'id__in' does not allow ManyToManyManager.")
+        return super(ManyToManyManager, self).filter(*args, **kwargs)
 
-    def __init__(self, main_client, model_name, endpoint, schema,
-                 objects=None, base_client=None):
-        self._client = getattr(main_client, model_name)
-        self._main_client = main_client
-        self._base_client = base_client
-        self._model_name = model_name
-        self._endpoint = endpoint
-        self._schema = schema
-        self._schema_store = self._base_client.schema(model_name)
-        self._base_url = self._main_client._store["base_url"]
-        self._fields = dict()  # TODO: set field attribute
-        self._strict_field = True
-        self.objects = objects or Manager(self)  # TODO: LazyCall
+
+def parse_id(resouce_uri):
+    """ url parsing
+
+    :param resource_uri:
+    :rtype: str
+    :return: primary id
+    """
+    return resouce_uri.split("/")[::-1][1]
+
+
+class Response(object):
+    """ Proxy Model Class """
+    def __init__(self, model, response=None, url=None, **kwargs):
+        """
+        :param model: The Model
+        :param response: response from Client Library
+        """
+        self.__response = response or dict()
+        self._schema = model.schema()
+        self._to_many_class = kwargs.get("_to_many_class", ManyToManyManager)
+        self._to_one_class = kwargs.get("_to_one_class", self.__class__)
+        self._url = url
+        if url is None:
+            self.model = model(**self.__response)
+        else:
+            self.model = model
 
     def __repr__(self):
-        return "<{0}: {1}{2}>".format(self._model_name, self._endpoint,
-                                      " " + str(self._fields) if self._fields else "")
+        return "<{0}: {1} {2}>".format(
+            self.model._model_name, self._url or "", self.__response)
 
-    def __call__(self, **kwargs):
-        self._setattrs(**kwargs)  # TODO: LazyCall
+    def __getattr__(self, attr):
+        """ return Response Class """
+        if not attr in self._response:
+            raise AttributeError(attr)
+        elif not "related_type" in self._schema["fields"][attr]:
+            return self.__getitem__(attr)
 
-        klass = copy.deepcopy(self)
-        self._clear_fields()
-        self._fields = dict()
-        return klass
+        related_type = self._schema["fields"][attr]["related_type"]
+        model = self.model.clone(attr)
+        url = self._response[attr]
+        if related_type == "to_many":
+            return self._to_many_class(model=model, query={"id__in": [parse_id(u) for u in url]})
+        elif related_type == "to_one":
+            return self._to_one_class(model=model, url=url)
 
-    def _clear_fields(self, klass=None):
-        c = klass or self
-        for field in c._fields:
-            c.__delattr__(field)
+    def __getitem__(self, item):
+        if item in self._response:
+            return getattr(self.model, item)
+        else:
+            raise KeyError(item)
 
-    def _clone(self, model_name, **kwargs):
-        """ create `model_name` model """
-        s = self._base_client.schema()[model_name]
-        klass = self.__class__(self._main_client, model_name,
-                               s["list_endpoint"], s["schema"], base_client=self._base_client)
-        klass.__dict__.update(kwargs)
-        return klass
-
-    def _setattrs(self, **kwargs):
-        for field in kwargs:
-            self.__setattr__(field, kwargs[field])
-            if not field in self._fields:
-                raise FieldTypeError("'{0}' is an invalid keyword argument for this function"
-                                     .format(field))
+    def __contains__(self, attr):
+        if hasattr(self, "_response") is False:
+            return False
+        return attr in self._response
 
     def __setattr__(self, attr, value):
-        self._setfield(attr, value)
-        super(Model, self).__setattr__(attr, value)
+        if "model" in self.__dict__:
+            if hasattr(self, attr):
+                self.__response[attr] = value
+                setattr(self.model, attr, value)
+        super(Response, self).__setattr__(attr, value)
 
-    def _setfield(self, field, value):
-        if hasattr(self, "_schema_store"):
-            if field in self._schema_store["fields"]:
-                nullable = self._schema_store["fields"][field]["nullable"]
-                blank = self._schema_store["fields"][field]["blank"]
-                field_type = self._schema_store["fields"][field]["type"]
-                check_type = False
-                value_ = value
-                try:
-                    #  TODO: type check and convert value.
-                    if (nullable or blank) and not value:
-                        check_type = True
-#                        value_ = value
-                    elif field_type == "string":
-                        check_type = isinstance(value, (str, unicode))
-#                        value_ = value
-                    elif field_type == "integer":
-                        check_type = True  # "".isdigit(), isinstance(value, int)
-#                        value_ = value
-                    elif field_type == "datetime":
-                        check_type = True  # return isinstance(value, datetime)
-#                        value_ = value
-                    elif field_type == "related":
-                        check_type = True  # return isinstance(value, datetime)
-#                        value_ = value
-                    elif field_type == "boolean":
-                        check_type = True  # return isinstance(value, datetime)
-#                        value_ = value
-                except Exception:
-                    check_type = False
-                finally:
-                    if check_type is not True and self._strict_field is True:
-                        raise FieldTypeError(
-                            "Field Type Error: '{0}' is '{1}' type. ( Input '{2}:{3}' )"
-                                .format(field, field_type, value_, type(value_).__name__))
-                # set field
-                self._fields[field] = value_
-
-    def _get_field(self, field):
-        if field in self._schema_store["fields"]:
-#            nullable = self._schema_store["fields"][field]["nullable"]
-#            blank = self._schema_store["fields"][field]["blank"]
-            field_type = self._schema_store["fields"][field]["type"]
-            value = self._fields[field]
-            try:
-                if field_type == "string":
-                    return value
-                elif field_type == "integer":
-                    return value
-                elif field_type == "datetime":
-                    return value
-                elif field_type == "related":
-                    value_ = getattr(value, "resource_uri", value)
-                    related_type = self._schema_store["fields"][field]["related_type"]
-                    if related_type == "to_many":
-                        #  TODO: ManyToMany Manager  parent_obj.add(related_object)
-                        return value_ if isinstance(value_, (list, tuple)) else [value_]
-                    else:
-                        return value_
-                elif field_type == "boolean":
-                    return value
-                else:
-                    return value
-            except Exception:
-                if self._strict_field is True:
-                    raise FieldTypeError(
-                        "Field Type Error: '{0}' is '{1}' type. ( Input '{2}:{3}' )"
-                            .format(field, field_type, value, type(value).__name__))
-
-    def _get_fields(self):
-        fields = {}
-        for field in self._fields:
-            fields.update({field: self._get_field(field)})
-        return fields
-
-    def schema(self, *attrs):
-        """
-
-        * attrs example ::
-
-                >>> self.schema("fields")
-                # out fields schema
-                >>> self.schema("fields", "id")
-                # out id schema
-
-        :param tuple attrs:
-        :rtype: dict
-        :return: model schema
-        """
-        if attrs:
-            s = self._schema_store
-            for attr in attrs:
-                s = s[attr]
-            return s
-        else:
-            return self._schema_store
+    @property
+    def _response(self):
+        if "model" in self.__dict__:
+            if not self.__response:
+                client = getattr(self.model._main_client, self.model._model_name)
+                self.__response = client(parse_id(self._url)).get()
+                self.model = self.model(**self.__response)
+        return self.__response
 
     def save(self):
-        """ save
-
-        :rtype: NoneType
-        """
-        if hasattr(self, "id"):
-            self._client(self.id).put(self._get_fields())  # return bool
-        else:
-            self._setattrs(**self._client.post(self._get_fields()))
+        """ save saved response """
+        self.model.save()
 
     def delete(self):
-        """ delete
+        """ remove saved response """
+        self.model.delete()
+        self.__response = dict()
 
-        :rtype: NoneType
+
+def model_gen(**configs):
+    """ generate model
+
+    :param slumber main_client:
+    :param str model_name: resource name
+    :param str endpoint: endpoint url
+    :param str schema: schema url
+    :param bool strict_field: strict field and convert value in field. ( default: True )
+    :param Manager objects: Manager Class
+    :param Client objects: Client Class
+    """
+    class Model(object):
+        """ Inner Class
         """
-        assert hasattr(self, "id") is True, "{0} object can't be deleted because its {2} attribute \
-            is set to None.".format(self._model_name, self._schema_store["fields"]["id"]["type"])
-        self._client(self.id).delete()
-        self._clear_fields()
+        _client = getattr(configs.get("main_client"), configs.get("model_name"))
+        _main_client = configs.get("main_client")
+        _base_client = configs.get("base_client")
+        _model_name = configs.get("model_name")
+        _endpoint = configs.get("endpoint")
+        _schema = configs.get("schema")
+        _strict_field = configs.get("strict_field", True)
+        _schema_store = _base_client.schema(_model_name)
+        _base_url = _main_client._store["base_url"]
+        _fields = dict()  # TODO: set field attribute
+        objects = None
+
+        def __init__(self, **kwargs):
+            self._clear_fields()
+            self._setattrs(**kwargs)  # TODO: LazyCall
+
+        def __repr__(self):
+            return "<{0}: {1}{2}>".format(self._model_name, self._endpoint,
+                                          " " + str(self._fields) if self._fields else "")
+
+        def _clear_fields(self, klass=None):
+            c = klass or self
+            for field in c._fields:
+                c.__delattr__(field)
+            c._fields = dict()
+
+        def _setattrs(self, **kwargs):
+            for field in kwargs:
+                self.__setattr__(field, kwargs[field])
+                if not field in self._fields:
+                    raise FieldTypeError("'{0}' is an invalid keyword argument for this function"
+                                         .format(field))
+
+        def __setattr__(self, attr, value):
+            self._setfield(attr, value)
+
+        def _setfield(self, attr, value):
+            if hasattr(self, "_schema_store"):
+                if attr in self._schema_store["fields"]:
+                    nullable = self._schema_store["fields"][attr]["nullable"]
+                    blank = self._schema_store["fields"][attr]["blank"]
+                    field_type = self._schema_store["fields"][attr]["type"]
+                    check_type = False
+                    err = ""
+                    if self._strict_field is True:
+                        try:
+                            if (nullable or blank) and not value:
+                                check_type = True
+                            elif field_type == "string":
+                                check_type = isinstance(value, (str, unicode))
+                            elif field_type == "integer":
+                                if isinstance(value, (str, unicode)):
+                                    check_type = value.isdigit()
+                                elif isinstance(value, int):
+                                    check_type = True
+                            elif field_type == "datetime":
+                                if isinstance(value, (str, unicode)):
+                                    try:
+                                        value = datetime.strptime(value, "%Y-%m-%dT%H:%M:%S.%f")
+                                    except ValueError:
+                                        value = datetime.strptime(value, "%Y-%m-%dT%H:%M:%S")
+                                check_type = isinstance(value, datetime)
+                            elif field_type == "time":
+                                check_type = True
+                            elif field_type == "boolean":
+                                check_type = True
+                            if field_type == "related":
+                                check_type = True
+                        except Exception, err:
+                            check_type = False
+                        finally:
+                            if check_type is not True:
+                                raise FieldTypeError(
+                                    "'{0}' is '{1}' type. ( Input '{2}:{3}' ) {4}"
+                                        .format(attr, field_type, value, type(value).__name__, err))
+                    self._fields[attr] = value  # set field
+            super(Model, self).__setattr__(attr, value)
+
+        def _get_field(self, field):
+            if field in self._schema_store["fields"]:
+                field_type = self._schema_store["fields"][field]["type"]
+                value = self._fields[field]
+                if self._strict_field is True:
+                    try:
+                        if field_type == "string":
+                            pass
+                        elif field_type == "integer":
+                            pass  #  input safe
+                        elif field_type == "datetime":
+                            value = value.isoformat()
+                        elif field_type == "time":
+                            pass
+                        elif field_type == "boolean":
+                            pass
+                        else:
+                            pass
+                    except Exception:
+                        if self._strict_field is True:
+                            raise FieldTypeError(
+                                "'{0}' is '{1}' type. ( Input '{2}:{3}' )"
+                                    .format(field, field_type, value, type(value).__name__))
+                 #  TODO: ManyToMany Manager  parent_obj.add(related_object)
+                if field_type == "related":
+                    value = getattr(value, "resource_uri", value)
+                    if self._schema_store["fields"][field]["related_type"] == "to_many":
+                        if isinstance(value, (list, tuple)) is False:
+                            value = [value]
+                return value
+
+        def _get_fields(self):
+            fields = {}
+            for field in self._fields:
+                fields.update({field: self._get_field(field)})
+            return fields
+
+        @classmethod
+        def clone(cls, model_name=None):
+            """ create `model_name` model """
+            return cls._base_client._model_gen(model_name or cls._model_name)
+
+        @classmethod
+        def schema(cls, *attrs):
+            """
+
+            * attrs example ::
+
+                    >>> self.schema("fields")
+                    # out fields schema
+                    >>> self.schema("fields", "id")
+                    # out id schema
+
+            :param tuple attrs:
+            :rtype: dict
+            :return: model schema
+            """
+            if attrs:
+                s = cls._schema_store
+                for attr in attrs:
+                    s = s[attr]
+                return s
+            else:
+                return cls._schema_store
+
+        def save(self):
+            """ save
+
+            :rtype: NoneType
+            """
+            if hasattr(self, "id"):
+                self._client(self.id).put(self._get_fields())  # return bool
+            else:
+                self._setattrs(**self._client.post(self._get_fields()))
+
+        def delete(self):
+            """ delete
+
+            :rtype: NoneType
+            """
+            assert hasattr(self, "id") is True, "{0} object can't be deleted because its {2} attribute \
+                is set to None.".format(self._model_name, self._schema_store["fields"]["id"]["type"])
+            self._client(self.id).delete()
+            self._clear_fields()
+
+    Model.objects = configs.get("objects", Manager(Model))
+    return Model
 
 
 class SchemaStore(dict):
@@ -622,10 +641,17 @@ class Client(object):
 
     __metaclass__ = ClientMeta
 
-    def __init__(self, base_url, auth=None, client=None):
+    def __init__(self, base_url, auth=None, strict_field=True, client=None):
+        """
+
+        :param str base_url: e.p. http://api.base.biz/base/v1/
+        :param auth:
+        :param bool strict_field: strict field and convert value in field.
+        :param object client:
+        """
         self._main_client = (client or slumber.API)(base_url, auth)
         self._base_url = self._main_client._store["base_url"]
-        self._method_gen()
+        self._methods_gen(strict_field)
 
     def request(self, url, method="GET"):
         """ base requester
@@ -657,7 +683,7 @@ class Client(object):
         """
         if not model_name in self._schema_store:
             url = self._url_gen("{0}/schema/".format(model_name)) if model_name \
-                                                                  else self._base_url
+                                                                 else self._base_url
             self._schema_store[model_name] = self.request(url)
         return self._schema_store[model_name]
 
@@ -671,10 +697,13 @@ class Client(object):
         else:
             return url
 
-    def _method_gen(self, base_client=None):
-        base_client = base_client or copy.deepcopy(self)
-        s = self.schema()
-        for model_name in s:
-            setattr(self, model_name, Model(self._main_client, model_name,
-                    s[model_name]["list_endpoint"], s[model_name]["schema"],
-                    base_client=base_client))
+    def _methods_gen(self, strict_field=True):
+        for model_name in self.schema():
+            setattr(self, model_name, self._model_gen(model_name, strict_field))
+
+    def _model_gen(self, model_name, strict_field=True, base_client=None):
+        schema = self.schema()
+        return model_gen(
+            main_client=self._main_client, model_name=model_name,
+            endpoint=schema[model_name]["list_endpoint"], schema=schema[model_name]["schema"],
+            strict_field=strict_field, base_client=base_client or copy.deepcopy(self))
\ No newline at end of file
